"""
Define the django form elements for the user interface. 
"""

from netbox.forms import NetBoxModelForm
from utilities.forms.fields import SlugField
from utilities.forms.rendering import FieldSet
from .. models import contract_filter_model


class ContractFilterForm(NetBoxModelForm):

    slug = SlugField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'contractsubject',
            'directives',
            'action',
        ),
    )

    class Meta:
        model = contract_filter_model.ContractFilter
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'contractsubject',
            'directives',
            'action',
        )



"""
Define the tables
"""

import django_tables2 as tables
from netbox.tables import NetBoxTable, columns

from . import models

__all__ = (
    "ApplicationProfileTable",
    "EndPointGroupTable",
    "ContractTable",
    "ContractSubjectTable",
    "ContractFilterTable",
    "BridgeDomainTable",
    "L3OutTable",
    "DomainTable",
    "AAEPTable",
    "AAEPStaticBindingTable",
    "IPGTable",
    "IPGAssignementTable",
    "LinkLevelTable",
)


class ApplicationProfileTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    tenant = tables.Column(
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.ap_model.ApplicationProfile
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "tenant",
        )
        default_columns = (
            "name",
            "tenant",
        )


class EndPointGroupTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    applicationprofile = tables.Column(
        linkify=True
    )

    domains = columns.ManyToManyColumn(
        linkify_item = True,
    )

    contracts_consume = columns.ManyToManyColumn(
        linkify_item = True,
    )

    contracts_provide = columns.ManyToManyColumn(
        linkify_item = True,
    )

    bridgedomain = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.epg_model.EndPointGroup
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "applicationprofile",
            "domains",
            "contracts_consume",
            "contracts_provide",
            "bridgedomain",
            "subnets",
        )
        default_columns = (
            "name",
            "applicationprofile",
            "domains",
            "contracts_consume",
            "contracts_provide",
            "bridgedomain",
        )


class EndPointSecurityGroupTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    vrf = tables.Column(
        linkify=True
    )

    contracts_consume = columns.ManyToManyColumn(
        linkify_item = True,
    )

    contracts_provide = columns.ManyToManyColumn(
        linkify_item = True,
    )

    epgs_selector = columns.ManyToManyColumn(
        linkify=True
    )

    ip_subnets_selector = columns.ManyToManyColumn(
        linkify=True
    )

    tags_selector = columns.ManyToManyColumn(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.esg_model.EndPointSecurityGroup
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "applicationprofile",
            "vrf",
            "contracts_consume",
            "contracts_provide",
            "epgs_selector",
            "ip_subnets_selector",
            "tags_selector",
        )
        default_columns = (
            "name",
            "applicationprofile",
            "vrf",
            "contracts_consume",
            "contracts_provide",
            "epgs_selector",
            "ip_subnets_selector",
            "tags_selector",
        )


class ContractTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    vrfs_consume = columns.ManyToManyColumn(
        linkify_item = True,
    )

    vrfs_provide = columns.ManyToManyColumn(
        linkify_item = True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.contract_model.Contract
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "scope",
            "qos_class",
            "target_dscp",
            "vrfs_consume",
            "vrfs_provide",
        )
        default_columns = (
            "name",
            "scope",
        )


class VrfContractTabTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.contract_model.Contract
        fields = (
            "name",
            "slug",
            "description",
            "comments",
        )
        default_columns = (
            "name",
            "scope",
        )


class ContractSubjectTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    contract = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.contract_subject_model.ContractSubject
        fields = (
            "name",
            "slug",
            "contract",
            "description",
            "comments",
            "target_dscp",
            "qos_priority",
            "apply_both_directions",
            "reverse_filter_ports",
        )
        default_columns = (
            "name",
        )


class ContractFilterTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    contractsubject = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.contract_filter_model.ContractFilter
        fields = (
            "name",
            "slug",
            "contractsubject",
            "description",
            "comments",
            "directives",
            "action",
        )
        default_columns = (
            "name",
        )


class ContractFilterEntryTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    contractfilter = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.contract_filter_entry_model.ContractFilterEntry
        fields = (
            "name",
            "slug",
            "contractfilter",
            "description",
            "comments",
            "ether_type",
            "ip_protocol",
            "arp_flag",
            "sport_from",
            "sport_to",
            "dport_from",
            "dport_to",
        )
        default_columns = (
            "name",
            "ether_type",
        )


class BridgeDomainTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    vrf = tables.Column(
        linkify=True
    )

    subnets = columns.ManyToManyColumn(
        linkify_item = True,
    )

    l3outs = columns.ManyToManyColumn(
        linkify_item = True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.bd_model.BridgeDomain
        fields = (
            "name",
            "slug",
            "description",
            "comments",            
            "vrf",
            "l2_unknown_unicast",
            "arp_flooding",
            "unicast_routing",
            "subnets",
            "l3outs",
            "comments",
        )
        default_columns = (
            "name",
            "vrf",
            "l2_unknown_unicast",
            "arp_flooding",
            "unicast_routing",
            "subnets",
            "l3outs",
        )


class L3OutTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    vrf = tables.Column(
        linkify=True
    )

    domains = columns.ManyToManyColumn(
        linkify_item = True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.l3out_model.L3Out
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "vrf",
            "domains",
        )
        default_columns = (
            "name",
            "vrf",
            "domains",            
        )


class DomainTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    vlan_pool = tables.Column(
        linkify = True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.domain_model.Domain
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "domain_type",
            "vlan_pool",
            "pool_allocation_mode",
        )
        default_columns = (
            "name",
            "domain_type",
            "vlan_pool",
            "pool_allocation_mode",
        )


class AAEPTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    domains = columns.ManyToManyColumn(
        linkify_item = True,
    )

    epg = columns.ManyToManyColumn(
        linkify_item = True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.aaep_model.AAEP
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "infrastructure_vlan",
            "domains",
        )
        default_columns = (
            "name",
            "domains",
        )


class AAEPStaticBindingTable(NetBoxTable):

    tenant = tables.Column(
        linkify=True
    )

    applicationprofile = tables.Column(
        linkify=True
    )

    epg = tables.Column(
        linkify = True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.aaep_model.AAEPStaticBinding
        fields = (
            "tenant",
            "applicationprofile",
            "epg",
            "encap",
            "mode",
        )
        default_columns = (
            "tenant",
            "applicationprofile",
            "epg",
            "encap",
            "mode",
        )


class IPGTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    aaep = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.ipg_model.PolicyGroup
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "aaep",
        )
        default_columns = (
            "name",
            "aaep",
        )


class IPGAssignementTable(NetBoxTable):

    ipg = tables.Column(
        linkify=True
    )

    device = tables.Column(
        linkify=True
    )

    interface = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        model = models.ipg_model.PolicyGroupAssignement
        fields = (
            "ipg",
            "device",
            "interface",
        )
        default_columns = (
            "ipg",
            "device",
            "interface",            
        )


class LinkLevelTable(NetBoxTable):

    name = tables.Column(
        linkify=True
    )

    tenant = tables.Column(
        linkify=True,
    )

    class Meta(NetBoxTable.Meta):
        model = models.policy_link_level_model.LinkLevel
        fields = (
            "name",
            "slug",
            "description",
            "comments",
            "negotiation",
            "speed",
        )
        default_columns = (
            "name",
            "negotiation",
            "speed",
        )



from netbox.plugins import PluginMenu, PluginMenuButton, PluginMenuItem

ap_menu_items = (
    PluginMenuItem(
        link='plugins:netbox_aci:applicationprofile_list',
        link_text='Application Profiles',
        permissions=['netbox_aci.view_applicationprofile'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:applicationprofile_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_applicationprofile'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_aci:endpointgroup_list',
        link_text='Application EPGs',
        permissions=['netbox_aci.view_endpointgroup'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:endpointgroup_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_endpointgroup'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_aci:endpointsecuritygroup_list',
        link_text='Endpoint Security Groups',
        permissions=['netbox_aci.view_endpointsecuritygroup'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:endpointsecuritygroup_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_endpointsecuritygroup'],
            ),
        ),
    ),
)

contract_menu_items = (
    PluginMenuItem(
        link='plugins:netbox_aci:contract_list',
        link_text='Standard',
        permissions=['netbox_aci.view_contract'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:contract_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_contract'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_aci:contractfilter_list',
        link_text='Filters',
        permissions=['netbox_aci.view_contract'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:contractfilter_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_contract'],
            ),
        ),
    ),
)

nwtworking_menu_items = (
    PluginMenuItem(
        link='plugins:netbox_aci:bridgedomain_list',
        link_text='Bridge Domains',
        permissions=['netbox_aci.view_bridgedomain'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:bridgedomain_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_bridgedomain'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_aci:l3out_list',
        link_text='L3Outs',
        permissions=['netbox_aci.view_l3out'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:l3out_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_l3out'],
            ),
        ),
    ),
)

fabric_menu_items = (
    PluginMenuItem(
        link='plugins:netbox_aci:domain_list',
        link_text='Domains',
        permissions=['netbox_aci.view_domain'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:domain_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_domain'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_aci:aaep_list',
        link_text='Attachable Access Entity Profiles',
        permissions=['netbox_aci.view_aaep'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:aaep_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_aaep'],
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_aci:policygroup_list',
        link_text='Policy Groups',
        permissions=['netbox_aci.view_policygroup'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:policygroup_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_policygroup'],
            ),
        ),
    ),
)

policy_menu_items = (
    PluginMenuItem(
        link='plugins:netbox_aci:linklevel_list',
        link_text='Link Levels',
        permissions=['netbox_aci.view_linklevel'],
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_aci:linklevel_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
                permissions=['netbox_aci.add_linklevel'],
            ),
        ),
    ),
)

menu = PluginMenu(
    label = 'ACI',
    groups=(("Application Profiles", ap_menu_items),
            ("Contracts", contract_menu_items),
            ("Networking", nwtworking_menu_items),
            ("Fabric", fabric_menu_items),
            ("Policies", policy_menu_items),
            ),
    icon_class='mdi mdi-router'
)



"""
Define the django model.
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import ContractFilterEntryEtherTypeChoices, ContractFilterEntryIPProtocolChoices, ContractFilterEntryARPFlagChoices
from . default_model import ACIDefault
from . contract_filter_model import ContractFilter

__all__ = (
    "ContractFilterEntry",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)

port_validation = RegexValidator(
    r"^(1|6[0-5][0-5][0-3][0-5]|[1-5][0-9][0-9][0-9][0-9]|[1-9][0-9]{0,3})$",
    "Port must be in the form 1-65535"
)


class ContractFilterEntry(ACIDefault):
    """
    This class definition defines a Django model for an entry within a contract filter.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=50,
        unique=True,
        validators=[input_validation],
    )

    contractfilter = models.ForeignKey(
        ContractFilter,
        on_delete=models.CASCADE,
        related_name="entry_filter",
        blank=True,
        null=True,
    )

    ether_type = models.CharField(
        choices=ContractFilterEntryEtherTypeChoices,
        blank=True,
    )

    ip_protocol = models.CharField(
        choices=ContractFilterEntryIPProtocolChoices,
        blank=True,
    )

    arp_flag = models.CharField(
        choices=ContractFilterEntryARPFlagChoices,
        blank=True,
    )

    sport_from = models.CharField(
        max_length=5,
        validators=[port_validation],
        blank=True,
        null=True,
        help_text=("source port 1-65535")
    )

    sport_to = models.CharField(
        max_length=5,
        validators=[port_validation],
        blank=True,
        null=True,
        help_text=("source port 1-65535")
    )

    dport_from = models.CharField(
        max_length=5,
        validators=[port_validation],
        blank=True,
        null=True,
        help_text=("destination port 1-65535")
    )

    dport_to = models.CharField(
        max_length=5,
        validators=[port_validation],
        blank=True,
        null=True,
        help_text=("destination port 1-65535")
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Contract Filter Entry"
        verbose_name_plural = "Contract Filter Entries"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:contractfilterentry', args=[self.pk])



"""
Define the django model.
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import DomainTypeChoices, PoolAllocationModeChoices
from . default_model import ACIDefault

__all__ = (
    "Domain",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class Domain(ACIDefault):
    """
    This class definition defines a Django model for Physical and External Domains.
    """
    #Fields
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    domain_type = models.CharField(
        choices=DomainTypeChoices,
        blank=True,
    )

    vlan_pool = models.ForeignKey(
        on_delete=models.PROTECT,
        to='ipam.VLANGroup',
    )

    pool_allocation_mode = models.CharField(
        choices=PoolAllocationModeChoices,
        blank=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Domain"
        verbose_name_plural = "Domains"

    #Metadata
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:domain', args=[self.pk])



"""
Define the django form elements for the user interface. 
"""

from django import forms
from netbox.forms import NetBoxModelForm
from utilities.forms import get_field_value
from utilities.forms.fields import SlugField
from utilities.forms.rendering import FieldSet
from .. models import contract_filter_entry_model


class ContractFilterEntryForm(NetBoxModelForm):

    slug = SlugField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'contractfilter',
            'ether_type',
            'ip_protocol',
            'arp_flag',
            'sport_from',
            'sport_to',
            'dport_from',
            'dport_to',
        ),
    )

    class Meta:
        model = contract_filter_entry_model.ContractFilterEntry

        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'contractfilter',
            'ether_type',
            'ip_protocol',
            'arp_flag',
            'sport_from',
            'sport_to',
            'dport_from',
            'dport_to',            
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if get_field_value(self, 'ether_type'):

            if get_field_value(self, 'ether_type') in 'ip, ipv4, ipv6':
                del self.fields['arp_flag']
                self.fields['ether_type'] = forms.ChoiceField(
                    choices=[(get_field_value(self, 'ether_type'), get_field_value(self, 'ether_type'))],
                    required=False
                )

            if get_field_value(self, 'ether_type') in 'mpls_unicast, unspecified, trill, fcoe, mac_security':
                del self.fields['arp_flag']
                del self.fields['ip_protocol']
                del self.fields['sport_from']
                del self.fields['sport_to']
                del self.fields['dport_from']
                del self.fields['dport_to']
                self.fields['ether_type'] = forms.ChoiceField(
                    choices=[(get_field_value(self, 'ether_type'), get_field_value(self, 'ether_type'))],
                    required=False
                )

            if get_field_value(self, 'ether_type') in 'arp':
                del self.fields['ip_protocol']
                del self.fields['sport_from']
                del self.fields['sport_to']
                del self.fields['dport_from']
                del self.fields['dport_to']
                self.fields['ether_type'] = forms.ChoiceField(
                    choices=[("arp", "arp")],
                    required=False
                )

    def clean(self):
        super().clean()

        ip_protocol = self.cleaned_data.get("ip_protocol")
        sport_from = self.cleaned_data.get("sport_from")
        sport_to = self.cleaned_data.get("sport_to")
        dport_from = self.cleaned_data.get("dport_from")
        dport_to = self.cleaned_data.get("dport_to")

        if ip_protocol in 'egp, eigrp, icmp, icmpv6, igmp, igp, l2tp, ospf, pim, unspecified':

            if sport_from or sport_to or dport_from or dport_to:
                self.add_error("sport_from", "Ports needed for TCP/UDP")
                self.add_error("sport_to", "Ports needed for TCP/UDP")
                self.add_error("dport_from", "Ports needed for TCP/UDP")
                self.add_error("dport_to", "Ports needed for TCP/UDP")



"""
Import models scripts
"""

from . ap_form import *
from . epg_form import *
from . esg_form import *
from . contract_form import *
from . contract_subject_form import *
from . contract_filter_form import *
from . contract_filter_entry_form import *
from . bd_form import *
from . l3out_form import *
from . domain_form import *
from . aaep_form import *
from . ipg_form import *
from . policy_link_level_form import *



"""
Import models scripts
"""

from . ap_model import *
from . epg_model import *
from . esg_model import *
from . contract_model import *
from . contract_subject_model import *
from . contract_filter_model import *
from . contract_filter_entry_model import *
from . bd_model import *
from . l3out_model import *
from . domain_model import *
from . aaep_model import *
from . ipg_model import *
from . policy_link_level_model import *



from netbox.plugins import PluginTemplateExtension
#from . models import contract_model


class IpamVrfVzanyViewExtension(PluginTemplateExtension):
    model = "ipam.vrf"

    def buttons(self):
        return self.render("netbox_aci/button_vzany.html")

#    def full_width_page(self):
#        obj = self.context['object']
#        consume_contracts = contract_model.Contract.objects.filter(vrfs_consume=obj)
#        provide_contracts = contract_model.Contract.objects.filter(vrfs_provide=obj)
#
#        return self.render("netbox_aci/netbox_ipam_vrf_vzanyviewextension.html",
#                           extra_context=
#                           {"vrf_consume_table": consume_contracts,
#                            "vrf_provide_table": provide_contracts,
#                            })


class DcimInterfaceIpgViewExtension(PluginTemplateExtension):
    model = "dcim.interface"

    def buttons(self):
        return self.render("netbox_aci/button_ipg.html")


template_extensions = [IpamVrfVzanyViewExtension, DcimInterfaceIpgViewExtension]



"""
Define the django model.
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import ContractFilterDirectivesChoices, ContractFilterACtionChoices
from . default_model import ACIDefault
from . contract_subject_model import ContractSubject

__all__ = (
    "ContractFilter",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class ContractFilter(ACIDefault):
    """
    This class definition defines a Django model for a contract filter.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=50,
        unique=True,
        validators=[input_validation],
    )

    contractsubject = models.ForeignKey(
        ContractSubject,
        on_delete=models.CASCADE,
        related_name="filter_subject",
        blank=True,
        null=True,
    )

    directives = models.CharField(
        choices=ContractFilterDirectivesChoices,
        blank=True,
    )

    action = models.CharField(
        choices=ContractFilterACtionChoices,
        blank=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Contract Filter"
        verbose_name_plural = "Contract Filters"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:contractfilter', args=[self.pk])



# Generated by Django 5.0.9 on 2024-11-20 15:48

import django.core.validators
import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0191_module_bay_rebuild'),
        ('extras', '0121_customfield_related_object_filter'),
        ('ipam', '0070_vlangroup_vlan_id_ranges'),
        ('tenancy', '0015_contactassignment_rename_content_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationProfile',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('tenant', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='tenancy.tenant')),
            ],
            options={
                'verbose_name': 'Application Profile',
                'verbose_name_plural': 'Application Profiles',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='BridgeDomain',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('l2_unknown_unicast', models.CharField(blank=True, default='hardware_proxy')),
                ('arp_flooding', models.BooleanField(default=True)),
                ('unicast_routing', models.BooleanField(default=False)),
                ('subnets', models.ManyToManyField(blank=True, related_name='bd_subnet', to='ipam.ipaddress')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vrf', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ipam.vrf')),
            ],
            options={
                'verbose_name': 'Bridge Domain',
                'verbose_name_plural': 'Bridge Domains',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Contract',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('scope', models.CharField(blank=True)),
                ('qos_class', models.CharField(blank=True)),
                ('target_dscp', models.CharField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vrfs_consume', models.ManyToManyField(blank=True, related_name='vrf_contract_consume', to='ipam.vrf')),
                ('vrfs_provide', models.ManyToManyField(blank=True, related_name='vrf_contract_provide', to='ipam.vrf')),
            ],
            options={
                'verbose_name': 'Contract',
                'verbose_name_plural': 'Contracts',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ContractFilter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=50, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('directives', models.CharField(blank=True)),
                ('action', models.CharField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Contract Filter',
                'verbose_name_plural': 'Contract Filters',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ContractFilterEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=50, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('ether_type', models.CharField(blank=True)),
                ('ip_protocol', models.CharField(blank=True)),
                ('arp_flag', models.CharField(blank=True)),
                ('sport_from', models.CharField(blank=True, max_length=5, null=True, validators=[django.core.validators.RegexValidator('^(1|6[0-5][0-5][0-3][0-5]|[1-5][0-9][0-9][0-9][0-9]|[1-9][0-9]{0,3})$', 'Port must be in the form 1-65535')])),
                ('sport_to', models.CharField(blank=True, max_length=5, null=True, validators=[django.core.validators.RegexValidator('^(1|6[0-5][0-5][0-3][0-5]|[1-5][0-9][0-9][0-9][0-9]|[1-9][0-9]{0,3})$', 'Port must be in the form 1-65535')])),
                ('dport_from', models.CharField(blank=True, max_length=5, null=True, validators=[django.core.validators.RegexValidator('^(1|6[0-5][0-5][0-3][0-5]|[1-5][0-9][0-9][0-9][0-9]|[1-9][0-9]{0,3})$', 'Port must be in the form 1-65535')])),
                ('dport_to', models.CharField(blank=True, max_length=5, null=True, validators=[django.core.validators.RegexValidator('^(1|6[0-5][0-5][0-3][0-5]|[1-5][0-9][0-9][0-9][0-9]|[1-9][0-9]{0,3})$', 'Port must be in the form 1-65535')])),
                ('contractfilter', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='entry_filter', to='netbox_aci.contractfilter')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Contract Filter Entry',
                'verbose_name_plural': 'Contract Filter Entries',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ContractSubject',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=50, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('target_dscp', models.CharField(blank=True)),
                ('qos_priority', models.CharField(blank=True)),
                ('apply_both_directions', models.BooleanField(default=True)),
                ('reverse_filter_ports', models.BooleanField(default=True)),
                ('contract', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subject_contract', to='netbox_aci.contract')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Contract Subject',
                'verbose_name_plural': 'Contract Subjects',
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='contractfilter',
            name='contractsubject',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='filter_subject', to='netbox_aci.contractsubject'),
        ),
        migrations.CreateModel(
            name='Domain',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('domain_type', models.CharField(blank=True)),
                ('pool_allocation_mode', models.CharField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vlan_pool', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='ipam.vlangroup')),
            ],
            options={
                'verbose_name': 'Domain',
                'verbose_name_plural': 'Domains',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='AAEP',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('infrastructure_vlan', models.BooleanField(default=False)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('domains', models.ManyToManyField(blank=True, related_name='aaep_domain', to='netbox_aci.domain')),
            ],
            options={
                'verbose_name': 'AAEP',
                'verbose_name_plural': 'AAEPs',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='EndPointGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('applicationprofile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='netbox_aci.applicationprofile')),
                ('bridgedomain', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='netbox_aci.bridgedomain')),
                ('contracts_consume', models.ManyToManyField(blank=True, related_name='epg_contract_consume', to='netbox_aci.contract')),
                ('contracts_provide', models.ManyToManyField(blank=True, related_name='epg_contract_provide', to='netbox_aci.contract')),
                ('domains', models.ManyToManyField(blank=True, related_name='epg_domain', to='netbox_aci.domain')),
                ('subnets', models.ManyToManyField(blank=True, related_name='epg_subnet', to='ipam.ipaddress')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'End Point Group',
                'verbose_name_plural': 'End Point Groups',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='EndPointSecurityGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('applicationprofile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='netbox_aci.applicationprofile')),
                ('contracts_consume', models.ManyToManyField(blank=True, related_name='esg_contract_consume', to='netbox_aci.contract')),
                ('contracts_provide', models.ManyToManyField(blank=True, related_name='esg_contract_provide', to='netbox_aci.contract')),
                ('epgs_selector', models.ManyToManyField(blank=True, related_name='esg_epg_selector', to='netbox_aci.endpointgroup')),
                ('ip_subnets_selector', models.ManyToManyField(blank=True, related_name='esg_ip_subnet_selector', to='ipam.prefix')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('tags_selector', models.ManyToManyField(blank=True, related_name='esg_tag_selector', to='extras.tag')),
                ('vrf', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ipam.vrf')),
            ],
            options={
                'verbose_name': 'End Point Security Group',
                'verbose_name_plural': 'End Point Security Groups',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='L3Out',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('domains', models.ManyToManyField(blank=True, related_name='l3out_domain', to='netbox_aci.domain')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vrf', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ipam.vrf')),
            ],
            options={
                'verbose_name': 'L3 Out',
                'verbose_name_plural': 'L3 Outs',
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='bridgedomain',
            name='l3outs',
            field=models.ManyToManyField(blank=True, related_name='bd_l3out', to='netbox_aci.l3out'),
        ),
        migrations.CreateModel(
            name='LinkLevel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('negotiation', models.CharField(blank=True)),
                ('speed', models.CharField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Link Level Policy',
                'verbose_name_plural': 'Link Level Policies',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='PolicyGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('slug', models.SlugField(max_length=100, null=True, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100, unique=True, validators=[django.core.validators.RegexValidator('^[a-zA-Z0-9-_]+$', 'Only alphanumeric, hyphens, and underscores are allowed.')])),
                ('aaep', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='ipg_aaep', to='netbox_aci.aaep')),
                ('linklevel', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='ipg_linklevel', to='netbox_aci.linklevel')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Policy Group',
                'verbose_name_plural': 'Policy Groups',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='PolicyGroupAssignement',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('device', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='dcim.device')),
                ('interface', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='dcim.interface')),
                ('ipg', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ipgassignement', to='netbox_aci.policygroup')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name': 'Policy Group Assignement',
                'verbose_name_plural': 'Policy Groups Assignements',
                'ordering': ['ipg'],
            },
        ),
        migrations.CreateModel(
            name='AAEPStaticBinding',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('index', models.PositiveIntegerField(null=True)),
                ('encap', models.CharField(max_length=9, null=True, validators=[django.core.validators.RegexValidator('^vlan-(?:[1-9][0-9]{0,2}|[1-3][0-9]{3}|40[0-3][0-9]|409[0-4])$', 'Vlan must be in the form vlan-[1-4094]')])),
                ('mode', models.CharField()),
                ('aaep', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='aaepstaticbinding_aaep', to='netbox_aci.aaep')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('tenant', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='aaepstaticbinding_tenant', to='tenancy.tenant')),
                ('applicationprofile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='aaepstaticbinding_applicationprofile', to='netbox_aci.applicationprofile')),
                ('epg', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='aaepstaticbinding_epg', to='netbox_aci.endpointgroup')),
            ],
            options={
                'ordering': ['aaep', 'index'],
                'unique_together': {('aaep', 'index')},
            },
        ),
    ]



from django.db import models
from netbox.models import NetBoxModel


class ACIDefault(NetBoxModel):
    """
    This class definition defines a Django default model.
    """
    slug = models.SlugField(
        verbose_name=('slug'),
        max_length=100,
        unique=True,
        null=True,
    )

    description = models.CharField(
        max_length=100,
        blank=True,
    )

    comments = models.TextField(
        blank=True
    )

    class Meta:
        abstract = True



"""
Define the django model.
"""

from netbox.models import NetBoxModel
from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from . default_model import ACIDefault
from . import aaep_model, policy_link_level_model

__all__ = (
    "PolicyGroup",
    "PolicyGroupAssignement",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class PolicyGroup(ACIDefault):
    """
    This class definition defines a Django model for an Interface Policy Group.
    """
    #Fields
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    aaep = models.ForeignKey(
        aaep_model.AAEP,
        on_delete=models.PROTECT,
        related_name="ipg_aaep",
    )

    linklevel = models.ForeignKey(
        policy_link_level_model.LinkLevel,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name="ipg_linklevel",
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Policy Group"
        verbose_name_plural = "Policy Groups"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:policygroup', args=[self.pk])


class PolicyGroupAssignement(NetBoxModel):
    """
    This class definition defines a Django model for an Interface Policy Group.
    """
    #Fields
    ipg = models.ForeignKey(
        on_delete=models.CASCADE,
        to=PolicyGroup,
        verbose_name="Policy Group",
        related_name="ipgassignement",
    )

    device = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    interface = models.ForeignKey(
        to='dcim.Interface',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    #Metadata
    class Meta:
        ordering = ["ipg"]
        verbose_name = "Policy Group Assignement"
        verbose_name_plural = "Policy Groups Assignements"

    #Methods
    def __str__(self):
        return self.ipg.slug

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:policygroupassignement', args=[self.pk])



"""
Define the logic of the plugin.
"""

from django.shortcuts import render
from django.views.generic import View
from ipam.models import vlans
from netbox.views import generic
from utilities.views import GetRelatedModelsMixin, ViewTab, register_model_view

from .. import tables
from .. models import domain_model
from .. forms import domain_form

__all__ = (
    "DomainView",
    "DomainListView",
    "DomainEditView",
    "DomainDeleteView",
    "DomainBulkDeleteView",
    "DomainIpamVlangroupTabView",
)


class DomainView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = domain_model.Domain.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'vlan_pool': instance.vlan_pool,
            'related_models': self.get_related_models(request, instance),
        }


class DomainListView(generic.ObjectListView):
    queryset = domain_model.Domain.objects.all()
    table = tables.DomainTable


class DomainEditView(generic.ObjectEditView):
    queryset = domain_model.Domain.objects.all()
    form = domain_form.DomainForm
    default_return_url = 'plugins:netbox_aci:domain_list'


class DomainDeleteView(generic.ObjectDeleteView):
    queryset = domain_model.Domain.objects.all()


class DomainBulkDeleteView(generic.BulkDeleteView):
    queryset = domain_model.Domain.objects.all()
    table = tables.DomainTable


@register_model_view(vlans.VLANGroup, "domains")
class DomainIpamVlangroupTabView(View):
    tab = ViewTab(
        label="ACI Domain Assignements",
        badge=lambda obj: domain_model.Domain.objects.filter(
            vlan_pool=obj
        ).count(),
        hide_if_empty = True,
        permission="netbox_aci.view_domain",
    )

    def get(self, request, pk):
        group = vlans.VLANGroup.objects.get(pk=pk)
        pool = domain_model.Domain.objects.filter(vlan_pool=group)

        return render(
            request,
            "netbox_aci/domain_tab.html",
            context={
                "tab": self.tab,
                "object": group,
                "domain_table": pool,
            },
        )



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import epg_model
from .. forms import epg_form
from .. import filtersets

__all__ = (
    "EndPointGroupView",
    "EndPointGroupListView",
    "EndPointGroupEditView",
    "EndPointGroupDeleteView",
    "EndPointGroupBulkDeleteView",
)


class EndPointGroupView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = epg_model.EndPointGroup.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'domain_table': instance.domains.all(),
            'contract_consume_table': instance.contracts_consume.all(),
            'contract_provide_table': instance.contracts_provide.all(),
            'subnet_table': instance.subnets.all(),
            'related_models': self.get_related_models(request, instance),
        }


class EndPointGroupListView(generic.ObjectListView):
    queryset = epg_model.EndPointGroup.objects.all()
    table = tables.EndPointGroupTable
    filterset = filtersets.EndPointGroupListFilterSet
    filterset_form = epg_form.EndPointGroupFilterForm


class EndPointGroupEditView(generic.ObjectEditView):
    queryset = epg_model.EndPointGroup.objects.all()
    form = epg_form.EndPointGroupForm
    default_return_url = 'plugins:netbox_aci:endpointgroup_list'


class EndPointGroupDeleteView(generic.ObjectDeleteView):
    queryset = epg_model.EndPointGroup.objects.all()


class EndPointGroupBulkDeleteView(generic.BulkDeleteView):
    queryset = epg_model.EndPointGroup.objects.all()
    table = tables.EndPointGroupTable



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import bd_model
from .. forms import bd_form
from .. import filtersets

__all__ = (
    "BridgeDomainView",
    "BridgeDomainListView",
    "BridgeDomainEditView",
    "BridgeDomainDeleteView",
    "BridgeDomainBulkDeleteView",
)


class BridgeDomainView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = bd_model.BridgeDomain.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'vrf': instance.vrf,
            'subnet_table': instance.subnets.all(),
            'l3out_table': instance.l3outs.all(),
            'related_models': self.get_related_models(request, instance),
        }


class BridgeDomainListView(generic.ObjectListView):
    queryset = bd_model.BridgeDomain.objects.all()
    table = tables.BridgeDomainTable
    filterset = filtersets.BridgeDomainListFilterSet
    filterset_form = bd_form.BridgeDomainFilterForm


class BridgeDomainEditView(generic.ObjectEditView):
    queryset = bd_model.BridgeDomain.objects.all()
    form = bd_form.BridgeDomainForm
    default_return_url = 'plugins:netbox_aci:bridgedomain_list'


class BridgeDomainDeleteView(generic.ObjectDeleteView):
    queryset = bd_model.BridgeDomain.objects.all()


class BridgeDomainBulkDeleteView(generic.BulkDeleteView):
    queryset = bd_model.BridgeDomain.objects.all()
    table = tables.BridgeDomainTable



"""
Define the django form elements for the user interface. 
"""

from netbox.forms import NetBoxModelForm
from utilities.forms.fields import CommentField, SlugField
from utilities.forms.rendering import FieldSet
from .. models import ap_model


class ApplicationProfileForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'tenant',
        ),
    )

    class Meta:
        model = ap_model.ApplicationProfile
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'tenant',
        )



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import policy_link_level_model
from .. forms import policy_link_level_form

__all__ = (
    "LinkLevelView",
    "LinkLevelListView",
    "LinkLevelEditView",
    "LinkLevelDeleteView",
)


class LinkLevelView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = policy_link_level_model.LinkLevel.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'related_models': self.get_related_models(request, instance),
        }


class LinkLevelListView(generic.ObjectListView):
    queryset = policy_link_level_model.LinkLevel.objects.all()
    table = tables.LinkLevelTable


class LinkLevelEditView(generic.ObjectEditView):
    queryset = policy_link_level_model.LinkLevel.objects.all()
    form = policy_link_level_form.LinkLevelForm
    default_return_url = 'plugins:netbox_aci:linklevel_list'


class LinkLevelDeleteView(generic.ObjectDeleteView):
    queryset = policy_link_level_model.LinkLevel.objects.all()



"""
Define the django form elements for the user interface. 
"""

from django import forms
from django.db.models import Q
from ipam.models import IPAddress
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, SlugField, DynamicModelChoiceField
from utilities.forms.rendering import FieldSet
from .. models import ap_model, epg_model, domain_model, contract_model


class EndPointGroupForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    domains = forms.ModelMultipleChoiceField(
        queryset=domain_model.Domain.objects.filter(
            Q(domain_type='vmm') |
            Q(domain_type='physical')
        )
    )

    subnets = forms.ModelMultipleChoiceField(
        queryset=IPAddress.objects.filter(custom_field_data={"gateway": True}),
        required=False
    )

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'applicationprofile',
            'domains',
            'contracts_consume',
            'contracts_provide',
            'bridgedomain',
            'subnets',
        ),
    )

    class Meta:
        model = epg_model.EndPointGroup
        fields = (
            'name',
            'slug',
            'description',
            'comments',            
            'applicationprofile',
            'domains',
            'contracts_consume',
            'contracts_provide',
            'bridgedomain',
            'subnets',
        )


class EndPointGroupFilterForm(NetBoxModelFilterSetForm):

    model = epg_model.EndPointGroup

    applicationprofile = DynamicModelChoiceField(
        queryset=ap_model.ApplicationProfile.objects.all(),
        required=False
    )

    domains = forms.ModelMultipleChoiceField(
        queryset=domain_model.Domain.objects.all(),
        required=False
    )

    contracts_consume = forms.ModelMultipleChoiceField(
        queryset=contract_model.Contract.objects.all(),
        required=False
    )

    contracts_provide = forms.ModelMultipleChoiceField(
        queryset=contract_model.Contract.objects.all(),
        required=False
    )



"""
Define the django form elements for the user interface. 
"""

from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, SlugField
from utilities.forms.rendering import FieldSet
from .. models import aaep_model, domain_model

class AAEPForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'infrastructure_vlan',
            'domains',
        ),
    )

    class Meta:
        model = aaep_model.AAEP
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'infrastructure_vlan',
            'domains',
        )


class AAEPStaticBindingForm(NetBoxModelForm):

    comments = CommentField()

    fieldsets = (
        FieldSet(
            'aaep',
            'tenant',
            'applicationprofile',
            'epg',
            'encap',
            'mode',
        ),
    )

    class Meta:
        model = aaep_model.AAEPStaticBinding
        fields = (
            'aaep',
            'tenant',
            'applicationprofile',
            'epg',
            'encap',
            'mode',
        )

    def clean(self):
        """
        Prevent duplicate entries
        """
        super().clean()

        tenant = self.cleaned_data.get("tenant")
        applicationprofile = self.cleaned_data.get("applicationprofile")
        epg = self.cleaned_data.get("epg")
        encap = self.cleaned_data.get("encap")
        mode = self.cleaned_data.get("mode")

        if tenant and applicationprofile and epg:
            if aaep_model.AAEPStaticBinding.objects.filter(mode='access_untagged').exists():
                self.add_error("mode", "Duplicate entry")
            if aaep_model.AAEPStaticBinding.objects.filter(mode='access_8021p').exists():
                if mode not in 'trunk':
                    self.add_error("mode", "Duplicate entry")
                if aaep_model.AAEPStaticBinding.objects.filter(encap=encap).exists():
                    self.add_error("encap", "Duplicate entry")
            if aaep_model.AAEPStaticBinding.objects.filter(mode='trunk').exists():
                if mode in 'access_untagged':
                    self.add_error("mode", "Duplicate entry")
            if aaep_model.AAEPStaticBinding.objects.filter(encap=encap).exists():
                self.add_error("encap", "Duplicate entry")


class AAEPFilterForm(NetBoxModelFilterSetForm):

    model = aaep_model.AAEP

    domains = forms.ModelMultipleChoiceField(
        queryset=domain_model.Domain.objects.all(),
        required=False
    )



"""
Define the django form elements for the user interface. 
"""

from django import forms
from netbox.forms import NetBoxModelForm
from utilities.forms.fields import CommentField, SlugField
from utilities.forms.rendering import FieldSet
from .. models import domain_model


class DomainForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'domain_type',
            'vlan_pool',
            'pool_allocation_mode',
        ),
    )

    class Meta:
        model = domain_model.Domain

        fields = (
            'name',
            'slug',
            'description',            
            'comments',
            'domain_type',
            'vlan_pool',
            'pool_allocation_mode',
        )



"""
Define django models
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from . default_model import ACIDefault
from . import ap_model, contract_model, bd_model, domain_model

__all__ = (
    "EndPointGroup",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class EndPointGroup(ACIDefault):
    """
    This class definition defines a Django model for an EndPointGroup.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    applicationprofile = models.ForeignKey(
        ap_model.ApplicationProfile,
        on_delete=models.PROTECT,
        null=True,
    )

    domains = models.ManyToManyField(
        domain_model.Domain,
        related_name = "epg_domain",
        blank=True,
    )

    contracts_consume = models.ManyToManyField(
        contract_model.Contract,
        related_name = "epg_contract_consume",
        blank=True,
    )

    contracts_provide = models.ManyToManyField(
        contract_model.Contract,
        related_name="epg_contract_provide",
        blank=True,
    )

    bridgedomain = models.ForeignKey(
        bd_model.BridgeDomain,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    subnets = models.ManyToManyField(
        to='ipam.IPAddress',
        related_name = "epg_subnet",
        blank=True,
    )


    #Metadata    
    class Meta:
        """
        This class definition is for a Django model's metadata,
        which is specifically for a EndPointGroup model.
        """
        ordering = ["name"]
        verbose_name = "End Point Group"
        verbose_name_plural = "End Point Groups"

    #Methods
    def __str__(self):
        """
        This is a special method in Python classes that returns a string
        representation of an object. In this case, when you try to print
        the object or convert an instance of the class to a string,
        it will return the value of the name attribute.

        For example, if you have a EndPointGroup object with name
        "My EndPointGroup", printing it will output "My EndPointGroup".

        This method is defined in epg_model:EndPointGroup.__stract.__str__

        Returns:
            (str) A string representation of the EndPointGroup instance.
        """
        return self.name

    def get_absolute_url(self):
        """
        This method defines a method named get_absolute_url in a Python class.
        When called, it returns the absolute URL for the current instance
        of the EndPointGroup model of the class.
        The URL is generated using the reverse function, which takes
        the name of a URL pattern and a list of arguments. In this case,
        the URL pattern is 'plugins:netbox_aci:endpointgroup' and the argument
        is [self.pk], which is the primary key of the current instance.

        Returns:
            (str) An absolute URL for the current instance of the EndPointGroup model.
        """
        return reverse('plugins:netbox_aci:endpointgroup', args=[self.pk])



"""
Define django models
"""

from netbox.models import NetBoxModel
from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import AAEPStaticBindingModeChoices
from . default_model import ACIDefault
from . import domain_model, ap_model, epg_model

__all__ = (
    "AAEP",
    "AAEPStaticBinding",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)

vid_validation = RegexValidator(
    r"^vlan-(?:[1-9][0-9]{0,2}|[1-3][0-9]{3}|40[0-3][0-9]|409[0-4])$",
    "Vlan must be in the form vlan-[1-4094]",
)


class AAEP(ACIDefault):
    """
    This class definition defines a Django model for an Attachable Access Entity Profile.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    infrastructure_vlan = models.BooleanField(
        default=False,
    )

    domains = models.ManyToManyField(
        domain_model.Domain,
        related_name = "aaep_domain",
        blank=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "AAEP"
        verbose_name_plural = "AAEPs"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:aaep', args=[self.pk])


class AAEPStaticBinding(NetBoxModel):
    """
    This class definition defines a Django model for an Static Binding in anAttachable Access Entity Profile.
    """
    aaep = models.ForeignKey(
        AAEP,
        on_delete=models.CASCADE,
        related_name="aaepstaticbinding_aaep",
    )

    index = models.PositiveIntegerField(
        null=True
    )

    tenant = models.ForeignKey(
        to='tenancy.Tenant',
        on_delete=models.CASCADE,
        related_name="aaepstaticbinding_tenant",
        null=True,
    )

    applicationprofile = models.ForeignKey(
        ap_model.ApplicationProfile,
        on_delete=models.CASCADE,
        related_name="aaepstaticbinding_applicationprofile",
        null=True,
    )

    epg = models.ForeignKey(
        epg_model.EndPointGroup,
        on_delete=models.CASCADE,
        related_name="aaepstaticbinding_epg",
        null=True,
    )

    encap = models.CharField(
        max_length=9,
        validators=[vid_validation],
        null=True,
        help_text=("Prefix vlan- followed by a numeric VLAN ID (1-4094)")
    )

    mode = models.CharField(
        choices=AAEPStaticBindingModeChoices,
    )

    #Metadata
    class Meta:
        ordering = ["aaep", "index"]
        unique_together = ["aaep", "index"]

    #Methods
    def __str__(self):
        return self.aaep.slug

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:aaepstaticbinding', args=[self.pk])



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import l3out_model
from .. forms import l3out_form

__all__ = (
    "L3OutView",
    "L3OutListView",
    "L3OutEditView",
    "L3OutDeleteView",
    "L3OutBulkDeleteView",
)


class L3OutView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = l3out_model.L3Out.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'domain_table': instance.domains.all(),
            'vrf': instance.vrf,
            'related_models': self.get_related_models(request, instance),
        }


class L3OutListView(generic.ObjectListView):
    queryset = l3out_model.L3Out.objects.all()
    table = tables.L3OutTable


class L3OutEditView(generic.ObjectEditView):
    queryset = l3out_model.L3Out.objects.all()
    form = l3out_form.L3OutForm
    default_return_url = 'plugins:netbox_aci:l3out_list'


class L3OutDeleteView(generic.ObjectDeleteView):
    queryset = l3out_model.L3Out.objects.all()


class L3OutBulkDeleteView(generic.BulkDeleteView):
    queryset = l3out_model.L3Out.objects.all()
    table = tables.L3OutTable



"""
Creates API endpoint URLs for the plugin.
"""

from netbox.api.routers import NetBoxRouter

from . import views

app_name = "netbox_aci"

router = NetBoxRouter()
router.register("applicationprofile", views.ApplicationProfileViewSet)
router.register("endpointgroup", views.EndPointGroupViewSet)
router.register("endpointsecuritygroup", views.EndPointSecurityGroupViewSet)
router.register("contract", views.ContractViewSet)
router.register("contractsubject", views.ContractSubjectViewSet)
router.register("contractfilter", views.ContractFilterViewSet)
router.register("contractfilterentry", views.ContractFilterEntryViewSet)
router.register("bridgedomain", views.BridgeDomainViewSet)
router.register("l3out", views.L3OutViewSet)
router.register("domain", views.DomainViewSet)
router.register("aaep", views.AAEPViewSet)
router.register("aaepstaticbinding", views.AAEPStaticBindingViewSet)
router.register("policygroup", views.PolicyGroupViewSet)
router.register("policygroupassignement", views.PolicyGroupAssignementViewSet)
router.register("linklevel", views.LinkLevelViewSet)

urlpatterns = router.urls



"""
Define the logic of the plugin.
"""

from django.shortcuts import render
from django.views.generic import View
from tenancy.models import tenants
from netbox.views import generic
from utilities.views import GetRelatedModelsMixin, ViewTab, register_model_view

from .. import tables
from .. models import ap_model
from .. forms import ap_form

__all__ = (
    "ApplicationProfileView",
    "ApplicationProfileListView",
    "ApplicationProfileEditView",
    "ApplicationProfileDeleteView",
    "ApplicationProfileBulkDeleteView",
    "ApplicationProfileTenancyTenantTabView",
)


class ApplicationProfileView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = ap_model.ApplicationProfile.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'related_models': self.get_related_models(request, instance),
        }


class ApplicationProfileListView(generic.ObjectListView):
    queryset = ap_model.ApplicationProfile.objects.all()
    table = tables.ApplicationProfileTable


class ApplicationProfileEditView(generic.ObjectEditView):
    queryset = ap_model.ApplicationProfile.objects.all()
    form = ap_form.ApplicationProfileForm
    default_return_url = 'plugins:netbox_aci:applicationprofile_list'


class ApplicationProfileDeleteView(generic.ObjectDeleteView):
    queryset = ap_model.ApplicationProfile.objects.all()


class ApplicationProfileBulkDeleteView(generic.BulkDeleteView):
    queryset = ap_model.ApplicationProfile.objects.all()
    table = tables.ApplicationProfileTable


@register_model_view(tenants.Tenant, "applicationprofiles")
class ApplicationProfileTenancyTenantTabView(View):
    tab = ViewTab(
        label="ACI Applicationprofile Assignements",
        badge=lambda obj: ap_model.ApplicationProfile.objects.filter(
            tenant=obj
        ).count(),
        hide_if_empty = True,
        permission="netbox_aci.view_applicationprofile",
    )

    def get(self, request, pk):
        tnt = tenants.Tenant.objects.get(pk=pk)
        anp = ap_model.ApplicationProfile.objects.filter(tenant=tnt)

        return render(
            request,
            "netbox_aci/applicationprofile_tab.html",
            context={
                "tab": self.tab,
                "object": tnt,
                "anp_table": anp,
            },
        )



"""
Define django models
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from . default_model import ACIDefault
from . import ap_model, contract_model, epg_model

__all__ = (
    "EndPointSecurityGroup",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class EndPointSecurityGroup(ACIDefault):
    """
    This class definition defines a Django model for an EndPointSecurityGroup.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    applicationprofile = models.ForeignKey(
        ap_model.ApplicationProfile,
        on_delete=models.PROTECT,
        null=True,
    )

    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    contracts_consume = models.ManyToManyField(
        contract_model.Contract,
        related_name = "esg_contract_consume",
        blank=True,
    )

    contracts_provide = models.ManyToManyField(
        contract_model.Contract,
        related_name="esg_contract_provide",
        blank=True,
    )

    epgs_selector = models.ManyToManyField(
        epg_model.EndPointGroup,
        related_name="esg_epg_selector",
        blank=True,
    )

    ip_subnets_selector = models.ManyToManyField(
        to='ipam.Prefix',
        related_name = "esg_ip_subnet_selector",
        blank=True,
    )

    tags_selector = models.ManyToManyField(
        to='extras.Tag',
        related_name = "esg_tag_selector",
        blank=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "End Point Security Group"
        verbose_name_plural = "End Point Security Groups"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:endpointsecuritygroup', args=[self.pk])



from django.db.models import Count
from netbox.api.viewsets import NetBoxModelViewSet

from .. import models

from . serializers import (
    ApplicationProfileSerializer,
    EndPointGroupSerializer,
    ContractSerializer,
    ContractSubjectSerializer,
    ContractFilterSerializer,
    ContractFilterEntrySerializer,
    BridgeDomainSerializer,
    EndPointSecurityGroupSerializer,
    L3OutSerializer,
    DomainSerializer,
    AAEPSerializer,
    AAEPStaticBindingSerializer,
    PolicyGroupSerializer,
    PolicyGroupAssignementSerializer,
    LinkLevelSerializer,
    )


class ApplicationProfileViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django ApplicationProfile model & associates it to a view.
    """

    queryset = models.ap_model.ApplicationProfile.objects.all()
    serializer_class = ApplicationProfileSerializer


class EndPointGroupViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django EndPointGroup model & associates it to a view.
    """

    queryset = models.epg_model.EndPointGroup.objects.all()
    serializer_class = EndPointGroupSerializer


class ContractViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django Contract model & associates it to a view.
    """

    queryset = models.contract_model.Contract.objects.all()
    serializer_class = ContractSerializer


class ContractSubjectViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django ContractSubject model & associates it to a view.
    """

    queryset = models.contract_subject_model.ContractSubject.objects.all()
    serializer_class = ContractSubjectSerializer


class ContractFilterViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django ContractFilter model & associates it to a view.
    """

    queryset = models.contract_filter_model.ContractFilter.objects.all()
    serializer_class = ContractFilterSerializer


class ContractFilterEntryViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django ContractFilter model & associates it to a view.
    """

    queryset = models.contract_filter_entry_model.ContractFilterEntry.objects.all()
    serializer_class = ContractFilterEntrySerializer


class BridgeDomainViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django Contract model & associates it to a view.
    """

    queryset = models.bd_model.BridgeDomain.objects.all()
    serializer_class = BridgeDomainSerializer


class EndPointSecurityGroupViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django EndPointSecurityGroup model & associates it to a view.
    """

    queryset = models.esg_model.EndPointSecurityGroup.objects.all()
    serializer_class = EndPointSecurityGroupSerializer


class L3OutViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django L3Out model & associates it to a view.
    """

    queryset = models.l3out_model.L3Out.objects.all()
    serializer_class = L3OutSerializer


class DomainViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django Domain model & associates it to a view.
    """

    queryset = models.domain_model.Domain.objects.all()
    serializer_class = DomainSerializer


class AAEPViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django AAEP model & associates it to a view.
    """

    queryset = models.aaep_model.AAEP.objects.all()
    serializer_class = AAEPSerializer


class AAEPStaticBindingViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django AAEP static binding model & associates it to a view.
    """

    queryset = models.aaep_model.AAEPStaticBinding.objects.all()
    serializer_class = AAEPStaticBindingSerializer


class PolicyGroupViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django Interface Policy Group model & associates it to a view.
    """

    queryset = models.ipg_model.PolicyGroup.objects.all()
    serializer_class = PolicyGroupSerializer


class PolicyGroupAssignementViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django Interface Policy Group Assignement model & associates it to a view.
    """

    queryset = models.ipg_model.PolicyGroupAssignement.objects.all()
    serializer_class = PolicyGroupAssignementSerializer


class LinkLevelViewSet(NetBoxModelViewSet):
    """
    Defines the view set for the django Interface Link Level Policy & associates it to a view.
    """

    queryset = models.policy_link_level_model.LinkLevel.objects.all()
    serializer_class = LinkLevelSerializer



__version__ = '0.0.1'
__author__ = """Marc-Aurel Mohr-Lenné"""

from netbox.plugins import PluginConfig


class NetBoxAciConfig(PluginConfig):
    name = 'netbox_aci'
    verbose_name = 'NetBox ACI'
    description = 'Manage Cisco ACI in NetBox'
    version = __version__
    author = __author__
    base_url = 'aci'
    min_version = "4.1.3"
    max_version = "4.2.0"

config = NetBoxAciConfig



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import esg_model
from .. forms import esg_form
from .. import filtersets

__all__ = (
    "EndPointSecurityGroupView",
    "EndPointSecurityGroupListView",
    "EndPointSecurityGroupEditView",
    "EndPointSecurityGroupDeleteView",
    "EndPointSecurityGroupBulkDeleteView",
)


class EndPointSecurityGroupView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = esg_model.EndPointSecurityGroup.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'vrf': instance.vrf,
            'contract_provide_table': instance.contracts_provide.all(),
            'contract_consume_table': instance.contracts_consume.all(),
            'epg_selector_table': instance.epgs_selector.all(),
            'ip_subnet_selector_table': instance.ip_subnets_selector.all(),
            'tag_selector_table': instance.tags_selector.all(),
            'related_models': self.get_related_models(request, instance),
        }


class EndPointSecurityGroupListView(generic.ObjectListView):
    queryset = esg_model.EndPointSecurityGroup.objects.all()
    table = tables.EndPointSecurityGroupTable
    filterset = filtersets.EndPointSecurityGroupListFilterSet
    filterset_form = esg_form.EndPointSecurityGroupFilterForm

class EndPointSecurityGroupEditView(generic.ObjectEditView):
    queryset = esg_model.EndPointSecurityGroup.objects.all()
    form = esg_form.EndPointSecurityGroupForm
    default_return_url = 'plugins:netbox_aci:endpointsecuritygroup_list'


class EndPointSecurityGroupDeleteView(generic.ObjectDeleteView):
    queryset = esg_model.EndPointSecurityGroup.objects.all()


class EndPointSecurityGroupBulkDeleteView(generic.BulkDeleteView):
    queryset = esg_model.EndPointSecurityGroup.objects.all()
    table = tables.EndPointSecurityGroupTable



from netbox.views.generic import ObjectChangeLogView
from . import views
from . import models
from django.urls import include, path
from utilities.urls import get_model_urls

urlpatterns = (
    # Application Profile
    path('applicationprofile/',
         views.ap_view.ApplicationProfileListView.as_view(),
         name='applicationprofile_list'),
    path('applicationprofile/add/',
         views.ap_view.ApplicationProfileEditView.as_view(),
         name='applicationprofile_add'),
    path('applicationprofile/<int:pk>/',
         views.ap_view.ApplicationProfileView.as_view(),
         name='applicationprofile'),
    path('applicationprofile/<int:pk>/edit/',
         views.ap_view.ApplicationProfileEditView.as_view(),
         name='applicationprofile_edit'),
    path('applicationprofile/<int:pk>/delete/',
         views.ap_view.ApplicationProfileDeleteView.as_view(),
         name='applicationprofile_delete'),
    path('applicationprofile/delete/',
        views.ap_view.ApplicationProfileBulkDeleteView.as_view(),
        name="applicationprofile_bulk_delete"),
    path('applicationprofile/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='applicationprofile_changelog',
         kwargs={'model': models.ap_model.ApplicationProfile}),
    path('applicationprofile/<int:pk>/',
         include(get_model_urls('netbox_aci', 'applicationprofile'))),

    # Endpoint Group
    path('endpointgroup/',
         views.epg_view.EndPointGroupListView.as_view(),
         name='endpointgroup_list'),
    path('endpointgroup/add/',
         views.epg_view.EndPointGroupEditView.as_view(),
         name='endpointgroup_add'),
    path('endpointgroup/<int:pk>/',
         views.epg_view.EndPointGroupView.as_view(),
         name='endpointgroup'),
    path('endpointgroup/<int:pk>/edit/',
         views.epg_view.EndPointGroupEditView.as_view(),
         name='endpointgroup_edit'),
    path('endpointgroup/<int:pk>/delete/',
         views.epg_view.EndPointGroupDeleteView.as_view(),
         name='endpointgroup_delete'),
    path('endpointgroup/delete/',
        views.epg_view.EndPointGroupBulkDeleteView.as_view(),
        name="endpointgroup_bulk_delete"),
    path('endpointgroup/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='endpointgroup_changelog',
         kwargs={'model': models.epg_model.EndPointGroup}),
    path('endpointgroup/<int:pk>/',
         include(get_model_urls('netbox_aci', 'endpointgroup'))),

    # Endpoint Security Group
    path('endpointsecuritygroup/',
         views.esg_view.EndPointSecurityGroupListView.as_view(),
         name='endpointsecuritygroup_list'),
    path('endpointsecuritygroup/add/',
         views.esg_view.EndPointSecurityGroupEditView.as_view(),
         name='endpointsecuritygroup_add'),
    path('endpointsecuritygroup/<int:pk>/',
         views.esg_view.EndPointSecurityGroupView.as_view(),
         name='endpointsecuritygroup'),
    path('endpointsecuritygroup/<int:pk>/edit/',
         views.esg_view.EndPointSecurityGroupEditView.as_view(),
         name='endpointsecuritygroup_edit'),
    path('endpointsecuritygroup/<int:pk>/delete/',
         views.esg_view.EndPointSecurityGroupDeleteView.as_view(),
         name='endpointsecuritygroup_delete'),
    path('endpointsecuritygroup/delete/',
        views.esg_view.EndPointSecurityGroupBulkDeleteView.as_view(),
        name="endpointsecuritygroup_bulk_delete"),
    path('endpointsecuritygroup/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='endpointsecuritygroup_changelog',
         kwargs={'model': models.esg_model.EndPointSecurityGroup}),
    path('endpointsecuritygroup/<int:pk>/',
         include(get_model_urls('netbox_aci', 'endpointsecuritygroup'))),

    # Contract
    path('contract/',
         views.contract_view.ContractListView.as_view(),
         name='contract_list'),
    path('contract/add/',
         views.contract_view.ContractEditView.as_view(),
         name='contract_add'),
    path('contract/<int:pk>/',
         views.contract_view.ContractView.as_view(),
         name='contract'),
    path('contract/<int:pk>/edit/',
         views.contract_view.ContractEditView.as_view(),
         name='contract_edit'),
    path('contract/<int:pk>/delete/',
         views.contract_view.ContractDeleteView.as_view(),
         name='contract_delete'),
    path('contract/delete/',
        views.contract_view.ContractBulkDeleteView.as_view(),
        name="contract_bulk_delete"),
    path('contract/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='contract_changelog',
         kwargs={'model': models.contract_model.Contract}),
    path('contract/<int:pk>/',
         include(get_model_urls('netbox_aci', 'contract'))),

    # Contract Subject
    path('contractsubject/',
         views.contract_subject_view.ContractSubjectListView.as_view(),
         name='contractsubject_list'),
    path('contractsubject/add/',
         views.contract_subject_view.ContractSubjectEditView.as_view(),
         name='contractsubject_add'),
    path('contractsubject/<int:pk>/',
         views.contract_subject_view.ContractSubjectView.as_view(),
         name='contractsubject'),
    path('contractsubject/<int:pk>/edit/',
         views.contract_subject_view.ContractSubjectEditView.as_view(),
         name='contractsubject_edit'),
    path('contractsubject/<int:pk>/delete/',
         views.contract_subject_view.ContractSubjectDeleteView.as_view(),
         name='contractsubject_delete'),
    path('contractsubject/delete/',
        views.contract_subject_view.ContractSubjectBulkDeleteView.as_view(),
        name="contractsubject_bulk_delete"),
    path('contractsubject/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='contractsubject_changelog',
         kwargs={'model': models.contract_subject_model.ContractSubject}),
    path('contractsubject/<int:pk>/',
         include(get_model_urls('netbox_aci', 'contractsubject'))),

    # Contract Filter
    path('contractfilter/',
         views.contract_filter_view.ContractFilterListView.as_view(),
         name='contractfilter_list'),
    path('contractfilter/add/',
         views.contract_filter_view.ContractFilterEditView.as_view(),
         name='contractfilter_add'),
    path('contractfilter/<int:pk>/',
         views.contract_filter_view.ContractFilterView.as_view(),
         name='contractfilter'),
    path('contractfilter/<int:pk>/edit/',
         views.contract_filter_view.ContractFilterEditView.as_view(),
         name='contractfilter_edit'),
    path('contractfilter/<int:pk>/delete/',
         views.contract_filter_view.ContractFilterDeleteView.as_view(),
         name='contractfilter_delete'),
    path('contractfilter/delete/',
        views.contract_filter_view.ContractFilterBulkDeleteView.as_view(),
        name="contractfilter_bulk_delete"),
    path('contractfilter/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='contractfilter_changelog',
         kwargs={'model': models.contract_filter_model.ContractFilter}),
    path('contractfilter/<int:pk>/',
         include(get_model_urls('netbox_aci', 'contractfilter'))),

    # Contract Filter Entry
    path('contractfilterentry/',
         views.contract_filter_entry_view.ContractFilterEntryListView.as_view(),
         name='contractfilterentry_list'),
    path('contractfilterentry/add/',
         views.contract_filter_entry_view.ContractFilterEntryEditView.as_view(),
         name='contractfilterentry_add'),
    path('contractfilterentry/<int:pk>/',
         views.contract_filter_entry_view.ContractFilterEntryView.as_view(),
         name='contractfilterentry'),
    path('contractfilterentry/<int:pk>/edit/',
         views.contract_filter_entry_view.ContractFilterEntryEditView.as_view(),
         name='contractfilterentry_edit'),
    path('contractfilterentry/<int:pk>/delete/',
         views.contract_filter_entry_view.ContractFilterEntryDeleteView.as_view(),
         name='contractfilterentry_delete'),
    path('contractfilterentry/delete/',
        views.contract_filter_entry_view.ContractFilterEntryBulkDeleteView.as_view(),
        name="contractfilterentry_bulk_delete"),
    path('contractfilterentry/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='contractfilterentry_changelog',
         kwargs={'model': models.contract_filter_entry_model.ContractFilterEntry}),
    path('contractfilterentry/<int:pk>/',
         include(get_model_urls('netbox_aci', 'contractfilterentry'))),

    # Bridge Domain
    path('bridgedomain/',
         views.bd_view.BridgeDomainListView.as_view(),
         name='bridgedomain_list'),
    path('bridgedomain/add/',
         views.bd_view.BridgeDomainEditView.as_view(),
         name='bridgedomain_add'),
    path('bridgedomain/<int:pk>/',
         views.bd_view.BridgeDomainView.as_view(),
         name='bridgedomain'),
    path('bridgedomain/<int:pk>/edit/',
         views.bd_view.BridgeDomainEditView.as_view(),
         name='bridgedomain_edit'),
    path('bridgedomain/<int:pk>/delete/',
         views.bd_view.BridgeDomainDeleteView.as_view(),
         name='bridgedomain_delete'),
    path('bridgedomain/delete/',
        views.bd_view.BridgeDomainBulkDeleteView.as_view(),
        name="bridgedomain_bulk_delete"),
    path('bridgedomain/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='bridgedomain_changelog',
         kwargs={'model': models.bd_model.BridgeDomain}),
    path('bridgedomain/<int:pk>/',
         include(get_model_urls('netbox_aci', 'bridgedomain'))),

    # L3Out
    path('l3out/',
         views.l3out_view.L3OutListView.as_view(),
         name='l3out_list'),
    path('l3out/add/',
         views.l3out_view.L3OutEditView.as_view(),
         name='l3out_add'),
    path('l3out/<int:pk>/',
         views.l3out_view.L3OutView.as_view(),
         name='l3out'),
    path('l3out/<int:pk>/edit/',
         views.l3out_view.L3OutEditView.as_view(),
         name='l3out_edit'),
    path('l3out/<int:pk>/delete/',
         views.l3out_view.L3OutDeleteView.as_view(),
         name='l3out_delete'),
    path('l3out/delete/',
       views.l3out_view.L3OutBulkDeleteView.as_view(),
        name="l3out_bulk_delete"),
    path('l3out/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='l3out_changelog',
         kwargs={'model': models.l3out_model.L3Out}),
    path('l3out/<int:pk>/',
         include(get_model_urls('netbox_aci', 'l3out'))),

    # Domain
    path('domain/',
         views.domain_view.DomainListView.as_view(),
         name='domain_list'),
    path('domain/add/',
         views.domain_view.DomainEditView.as_view(),
         name='domain_add'),
    path('domain/<int:pk>/',
         views.domain_view.DomainView.as_view(),
         name='domain'),
    path('domain/<int:pk>/edit/',
         views.domain_view.DomainEditView.as_view(),
         name='domain_edit'),
    path('domain/<int:pk>/delete/',
         views.domain_view.DomainDeleteView.as_view(),
         name='domain_delete'),
    path('domain/delete/',
       views.domain_view.DomainBulkDeleteView.as_view(),
        name="domain_bulk_delete"),
    path('domain/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='domain_changelog',
         kwargs={'model': models.domain_model.Domain}),
    path('domain/<int:pk>/',
         include(get_model_urls('netbox_aci', 'domain'))),

    # AAEP
    path('aaep/',
         views.aaep_view.AAEPListView.as_view(),
         name='aaep_list'),
    path('aaep/add/',
         views.aaep_view.AAEPEditView.as_view(),
         name='aaep_add'),
    path('aaep/<int:pk>/',
         views.aaep_view.AAEPView.as_view(),
         name='aaep'),
    path('aaep/<int:pk>/edit/',
         views.aaep_view.AAEPEditView.as_view(),
         name='aaep_edit'),
    path('aaep/<int:pk>/delete/',
         views.aaep_view.AAEPDeleteView.as_view(),
         name='aaep_delete'),
    path('aaep/delete/',
        views.aaep_view.AAEPBulkDeleteView.as_view(),
        name="aaep_bulk_delete"),
    path('aaep/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='aaep_changelog',
         kwargs={'model': models.aaep_model.AAEP}),
    path('aaep/<int:pk>/',
         include(get_model_urls('netbox_aci', 'aaep'))),

    # AAEP STATIC BINDING
    path('aaepstaticbinding/',
         views.aaep_view.AAEPStaticBindingListView.as_view(),
         name='aaepstaticbinding_list'),
    path('aaepstaticbinding/add/',
         views.aaep_view.AAEPStaticBindingEditView.as_view(),
         name='aaepstaticbinding_add'),
    path('aaepstaticbinding/<int:pk>/',
         views.aaep_view.AAEPStaticBindingView.as_view(),
         name='aaepstaticbinding'),
    path('aaepstaticbinding/<int:pk>/edit/',
         views.aaep_view.AAEPStaticBindingEditView.as_view(),
         name='aaepstaticbinding_edit'),
    path('aaepstaticbinding/<int:pk>/delete/',
         views.aaep_view.AAEPStaticBindingDeleteView.as_view(),
         name='aaepstaticbinding_delete'),
    path('aaepstaticbinding/delete/',
        views.aaep_view.AAEPStaticBindingBulkDeleteView.as_view(),
        name="aaepstaticbinding_bulk_delete"),
    path('aaepstaticbinding/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='aaepstaticbinding_changelog',
         kwargs={'model': models.aaep_model.AAEPStaticBinding}),
    path('aaepstaticbinding/<int:pk>/',
         include(get_model_urls('netbox_aci', 'aaepstaticbinding'))),

    # Policy Group IPG
    path('policygroup/',
         views.ipg_view.PolicyGroupListView.as_view(),
         name='policygroup_list'),
    path('policygroup/add/',
         views.ipg_view.PolicyGroupEditView.as_view(),
         name='policygroup_add'),
    path('policygroup/<int:pk>/',
         views.ipg_view.PolicyGroupView.as_view(),
         name='policygroup'),
    path('policygroup/<int:pk>/edit/',
         views.ipg_view.PolicyGroupEditView.as_view(),
         name='policygroup_edit'),
    path('policygroup/<int:pk>/delete/',
         views.ipg_view.PolicyGroupDeleteView.as_view(),
         name='policygroup_delete'),
    path('policygroup/delete/',
        views.ipg_view.PolicyGroupBulkDeleteView.as_view(),
        name="policygroup_bulk_delete"),    
    path('policygroup/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='policygroup_changelog',
         kwargs={'model': models.ipg_model.PolicyGroup}),
    path('policygroup/<int:pk>/',
         include(get_model_urls('netbox_aci', 'policygroup'))),

    # Policy Group Assignement
    path('policygroupassignement/',
         views.ipg_view.PolicyGroupAssignementListView.as_view(),
         name='policygroupassignement_list'),
    path('policygroupassignement/add/',
         views.ipg_view.PolicyGroupAssignementEditView.as_view(),
         name='policygroupassignement_add'),
    path('policygroupassignement/<int:pk>/',
         views.ipg_view.PolicyGroupAssignementView.as_view(),
         name='policygroupassignement'),
    path('policygroupassignement/<int:pk>/edit/',
         views.ipg_view.PolicyGroupAssignementEditView.as_view(),
         name='policygroupassignement_edit'),
    path('policygroupassignement/<int:pk>/delete/',
         views.ipg_view.PolicyGroupAssignementDeleteView.as_view(),
         name='policygroupassignement_delete'),
    path('policygroupassignement/delete/',
        views.ipg_view.PolicyGroupAssignementBulkDeleteView.as_view(),
        name="policygroupassignement_bulk_delete"),
    path('policygroupassignement/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='policygroupassignement_changelog',
         kwargs={'model': models.ipg_model.PolicyGroupAssignement}),
    path('policygroupassignement/<int:pk>/',
         include(get_model_urls('netbox_aci', 'policygroupassignement'))),

    # Policy Link Level
    path('linklevel/',
         views.policy_link_level_view.LinkLevelListView.as_view(),
         name='linklevel_list'),
    path('linklevel/add/',
         views.policy_link_level_view.LinkLevelEditView.as_view(),
         name='linklevel_add'),
    path('linklevel/<int:pk>/',
         views.policy_link_level_view.LinkLevelView.as_view(),
         name='linklevel'),
    path('linklevel/<int:pk>/edit/',
         views.policy_link_level_view.LinkLevelEditView.as_view(),
         name='linklevel_edit'),
    path('linklevel/<int:pk>/delete/',
         views.policy_link_level_view.LinkLevelDeleteView.as_view(),
         name='linklevel_delete'),
    path('linklevel/<int:pk>/changelog/',
         ObjectChangeLogView.as_view(),
         name='linklevel_changelog',
         kwargs={'model': models.policy_link_level_model.LinkLevel}),
    path('linklevel/<int:pk>/',
         include(get_model_urls('netbox_aci', 'linklevel'))),
)


"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import aaep_model
from .. forms import aaep_form
from .. import filtersets

__all__ = (
    "AAEPView",
    "AAEPListView",
    "AAEPEditView",
    "AAEPDeleteView",
    "AAEPBulkDeleteView",
    "AAEPStaticBindingView",
    "AAEPStaticBindingListView",
    "AAEPStaticBindingEditView",
    "AAEPStaticBindingDeleteView",
    "AAEPStaticBindingBulkDeleteView",
)


class AAEPView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = aaep_model.AAEP.objects.all()

    def get_extra_context(self, request, instance):

        aaepstatic = instance.aaepstaticbinding_aaep.all()
        static_table = tables.AAEPStaticBindingTable(aaepstatic)
        return {
            'domain_table': instance.domains.all(),
            'static_table': static_table,
            'related_models': self.get_related_models(request, instance),
        }


class AAEPListView(generic.ObjectListView):
    queryset = aaep_model.AAEP.objects.all()
    table = tables.AAEPTable
    filterset = filtersets.AAEPListFilterSet
    filterset_form = aaep_form.AAEPFilterForm

class AAEPEditView(generic.ObjectEditView):
    queryset = aaep_model.AAEP.objects.all()
    form = aaep_form.AAEPForm
    default_return_url = 'plugins:netbox_aci:aaep_list'


class AAEPDeleteView(generic.ObjectDeleteView):
    queryset = aaep_model.AAEP.objects.all()


class AAEPBulkDeleteView(generic.BulkDeleteView):
    queryset = aaep_model.AAEP.objects.all()
    table = tables.AAEPTable


# Static Binding

class AAEPStaticBindingView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = aaep_model.AAEPStaticBinding.objects.all()


class AAEPStaticBindingListView(generic.ObjectListView):
    queryset = aaep_model.AAEPStaticBinding.objects.all()
    table = tables.AAEPStaticBindingTable
    actions = {'add': {'add'}}


class AAEPStaticBindingEditView(generic.ObjectEditView):
    queryset = aaep_model.AAEPStaticBinding.objects.all()
    form = aaep_form.AAEPStaticBindingForm
    default_return_url = 'plugins:netbox_aci:aaepstaticbinding_list'


class AAEPStaticBindingDeleteView(generic.ObjectDeleteView):
    queryset = aaep_model.AAEPStaticBinding.objects.all()


class AAEPStaticBindingBulkDeleteView(generic.BulkDeleteView):
    queryset = aaep_model.AAEPStaticBinding.objects.all()
    table = tables.AAEPStaticBindingTable



"""
Define django models
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from . default_model import ACIDefault
from . domain_model import Domain

__all__ = (
    "L3Out",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class L3Out(ACIDefault):
    """
    This class definition defines a Django model for an L3Out.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    domains = models.ManyToManyField(
        Domain,
        related_name = "l3out_domain",
        blank=True,
    )

    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )


    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "L3 Out"
        verbose_name_plural = "L3 Outs"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:l3out', args=[self.pk])



"""
Define the django model.
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import ContractScopeChoices, ContractQoSClassChoices, ContractTargetDSCPChoices
from . default_model import ACIDefault

__all__ = (
    "Contract",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class Contract(ACIDefault):
    """
    This class definition defines a Django model for a contract.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    scope = models.CharField(
        choices=ContractScopeChoices,
        blank=True,
    )

    qos_class = models.CharField(
        choices=ContractQoSClassChoices,
        blank=True,
    )

    target_dscp = models.CharField(
        choices=ContractTargetDSCPChoices,
        blank=True,
    )

    vrfs_consume = models.ManyToManyField(
        to='ipam.VRF',
        related_name = "vrf_contract_consume",
        blank=True,
    )

    vrfs_provide = models.ManyToManyField(
        to='ipam.VRF',
        related_name = "vrf_contract_provide",
        blank=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Contract"
        verbose_name_plural = "Contracts"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:contract', args=[self.pk])



from django.db.models import Q
from netbox.filtersets import NetBoxModelFilterSet
from . models import epg_model, esg_model, bd_model, aaep_model, ipg_model


class EndPointGroupListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = epg_model.EndPointGroup
        fields = ['id',
                  'name',
                  'description',
                  'applicationprofile',
                  'domains',
                  'contracts_consume',
                  'contracts_provide'
                  ]

    def search(self, queryset, name, value):
        query = (
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(applicationprofile__name__icontains=value) |
            Q(domains__name__icontains=value) |
            Q(contracts_consume__name__icontains=value) |
            Q(contracts_provide__name__icontains=value)
            )
        return queryset.filter(query)


class EndPointSecurityGroupListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = esg_model.EndPointSecurityGroup
        fields = ['id',
                  'name',
                  'description',
                  'applicationprofile',
                  'vrf',
                  'contracts_consume',
                  'contracts_provide',
                  'epgs_selector',
                  'ip_subnets_selector',
                  'tags_selector'
                  ]

    def search(self, queryset, name, value):
        query = (
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(applicationprofile__name__icontains=value) |
            Q(vrf__name__icontains=value) |
            Q(contracts_consume__name__icontains=value) |
            Q(contracts_provide__name__icontains=value) |
            Q(epgs_selector__name__icontains=value) |
            Q(ip_subnets_selector__name__icontains=value) |
            Q(tags_selector__name__icontains=value)
            )
        return queryset.filter(query)


class BridgeDomainListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = bd_model.BridgeDomain
        fields = ['id', 'name', 'description', 'vrf', 'l3outs']

    def search(self, queryset, name, value):
        query = (
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(vrf__name__icontains=value) |
            Q(l3outs__name__icontains=value)
            )
        return queryset.filter(query)


class AAEPListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = aaep_model.AAEP
        fields = ['id', 'name', 'description', 'domains']

    def search(self, queryset, name, value):
        query = (
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(domains__name__icontains=value)
            )
        return queryset.filter(query)


class PolicyGroupListFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = ipg_model.PolicyGroup
        fields = ['id', 'name', 'description', 'aaep']

    def search(self, queryset, name, value):
        query = (
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(aaep__name__icontains=value)
            )
        return queryset.filter(query)



"""
Define django models
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import BridgeDomainL2UUCChoices
from . default_model import ACIDefault
from . l3out_model import L3Out

__all__ = (
    "BridgeDomain",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class BridgeDomain(ACIDefault):
    """
    This class definition defines a Django model for a BridgeDomain.
    """
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    vrf = models.ForeignKey(
        to='ipam.VRF',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    l2_unknown_unicast = models.CharField(
        choices=BridgeDomainL2UUCChoices,
        default='hardware_proxy',
        blank=True,
    )

    arp_flooding = models.BooleanField(
        default=True,
    )

    unicast_routing = models.BooleanField(
        default=False,
    )

    subnets = models.ManyToManyField(
        to='ipam.IPAddress',
        related_name = "bd_subnet",
        blank=True,
    )

    l3outs = models.ManyToManyField(
        L3Out,
        related_name = "bd_l3out",
        blank=True,
    )


    #Metadata
    class Meta:
        """
        This class definition is for a Django model's metadata,
        which is specifically for a BridgeDomain model.
        """
        ordering = ["name"]
        verbose_name = "Bridge Domain"
        verbose_name_plural = "Bridge Domains"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:bridgedomain', args=[self.pk])



"""
Define the django form elements for the user interface. 
"""

from netbox.forms import NetBoxModelForm
from utilities.forms.fields import SlugField
from utilities.forms.rendering import FieldSet
from .. models import contract_subject_model


class ContractSubjectForm(NetBoxModelForm):

    slug = SlugField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'contract',
            'target_dscp',
            'qos_priority',
            'apply_both_directions',
            'reverse_filter_ports',
        ),
    )

    class Meta:
        model = contract_subject_model.ContractSubject
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'contract',
            'target_dscp',
            'qos_priority',
            'apply_both_directions',
            'reverse_filter_ports',
        )



from netbox.search import SearchIndex, register_search
from . models import aaep_model, ap_model, bd_model, contract_model, domain_model, epg_model, esg_model, ipg_model, l3out_model

@register_search
class AAEPIndex(SearchIndex):
    model = aaep_model.AAEP
    fields = (
        ('name', 100),
    )

@register_search
class ApplicationProfileIndex(SearchIndex):
    model = ap_model.ApplicationProfile
    fields = (
        ('name', 100),
    )

@register_search
class BridgeDomainIndex(SearchIndex):
    model = bd_model.BridgeDomain
    fields = (
        ('name', 100),
    )

@register_search
class ContractIndex(SearchIndex):
    model = contract_model.Contract
    fields = (
        ('name', 100),
    )

@register_search
class DomainIndex(SearchIndex):
    model = domain_model.Domain
    fields = (
        ('name', 100),
    )

@register_search
class EPGIndex(SearchIndex):
    model = epg_model.EndPointGroup
    fields = (
        ('name', 100),
    )

@register_search
class ESGIndex(SearchIndex):
    model = esg_model.EndPointSecurityGroup
    fields = (
        ('name', 100),
    )

@register_search
class IPGIndex(SearchIndex):
    model = ipg_model.PolicyGroup
    fields = (
        ('name', 100),
    )

@register_search
class L3OutIndex(SearchIndex):
    model = l3out_model.L3Out
    fields = (
        ('name', 100),
    )



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import contract_filter_entry_model
from .. forms import contract_filter_entry_form

__all__ = (
    "ContractFilterEntryView",
    "ContractFilterEntryListView",
    "ContractFilterEntryEditView",
    "ContractFilterEntryDeleteView",
    "ContractFilterEntryBulkDeleteView",
)


class ContractFilterEntryView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = contract_filter_entry_model.ContractFilterEntry.objects.all()

    def get_extra_context(self, request, instance):

        return {
            'related_models': self.get_related_models(request, instance),
        }


class ContractFilterEntryListView(generic.ObjectListView):
    queryset = contract_filter_entry_model.ContractFilterEntry.objects.all()
    table = tables.ContractFilterEntryTable


class ContractFilterEntryEditView(generic.ObjectEditView):
    queryset = contract_filter_entry_model.ContractFilterEntry.objects.all()
    form = contract_filter_entry_form.ContractFilterEntryForm
    default_return_url = 'plugins:netbox_aci:contractfilterentry_list'


class ContractFilterEntryDeleteView(generic.ObjectDeleteView):
    queryset = contract_filter_entry_model.ContractFilterEntry.objects.all()

class ContractFilterEntryBulkDeleteView(generic.BulkDeleteView):
    queryset = contract_filter_entry_model.ContractFilterEntry.objects.all()
    table = tables.ContractFilterEntryTable



from rest_framework import serializers
from netbox.api.serializers import NetBoxModelSerializer
from tenancy.api.serializers import TenantSerializer
from ipam.api.serializers import VLANGroupSerializer, VRFSerializer, IPAddressSerializer, PrefixSerializer
from extras.api.serializers import TagSerializer
from dcim.api.serializers import DeviceSerializer, InterfaceSerializer
from .. import models


class ApplicationProfileSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:applicationprofile-detail',
    )

    tenant = TenantSerializer(nested=True)

    class Meta:
        model = models.ap_model.ApplicationProfile
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'created',
            'last_updated',
            'url',
            'tenant',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class ContractSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:contract-detail',
    )

    vrfs_consume = VRFSerializer(nested=True, many=True, required=False, allow_null=True)
    vrfs_provide = VRFSerializer(nested=True, many=True, required=False, allow_null=True)

    class Meta:
        model = models.contract_model.Contract
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'scope',
            'qos_class',
            'target_dscp',
            'vrfs_consume',
            'vrfs_provide',
            'created',
            'last_updated',
            'url',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class ContractSubjectSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:contractsubject-detail',
    )

    contract = ContractSerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = models.contract_subject_model.ContractSubject
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'contract',
            'qos_priority',
            'target_dscp',
            'apply_both_directions',
            'reverse_filter_ports',            
            'created',
            'last_updated',
            'url',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class ContractFilterSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:contractfilter-detail',
    )

    contractsubject = ContractSubjectSerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = models.contract_filter_model.ContractFilter
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'contractsubject',
            'directives',
            'action',
            'created',
            'last_updated',
            'url',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class ContractFilterEntrySerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:contractfilterentry-detail',
    )

    contractfilter = ContractFilterSerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = models.contract_filter_entry_model.ContractFilterEntry
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'contractfilter',
            'ether_type',
            'ip_protocol',
            'arp_flag',
            'sport_from',
            'sport_to',
            'dport_from',
            'dport_to',
            'created',
            'last_updated',
            'url',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class L3OutSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:l3out-detail',
    )

    vrf = VRFSerializer(nested=True, required=False, allow_null=True)
    domains = VLANGroupSerializer(nested=True, many=True, required=False, allow_null=True)

    class Meta:
        model = models.l3out_model.L3Out
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',            
            'created',
            'last_updated',
            'url',
            'vrf',
            'domains',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class BridgeDomainSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:bridgedomain-detail',
    )

    vrf = VRFSerializer(nested=True, required=False, allow_null=True)
    subnets = IPAddressSerializer(nested=True, many=True, required=False, allow_null=True)
    l3outs = L3OutSerializer(nested=True, many=True, required=False, allow_null=True)

    class Meta:
        model = models.bd_model.BridgeDomain
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',            
            'vrf',
            'l2_unknown_unicast',
            'arp_flooding',
            'unicast_routing',
            'subnets',
            'l3outs',
            'created',
            'last_updated',
            'url',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class EndPointGroupSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:endpointgroup-detail',
    )

    applicationprofile = ApplicationProfileSerializer(nested=True)
    bridgedomain = BridgeDomainSerializer(nested=True, required=False, allow_null=True)
    subnets = IPAddressSerializer(nested=True, many=True, required=False, allow_null=True)
    domains = VLANGroupSerializer(nested=True, many=True, required=False, allow_null=True)
    contracts_provide = ContractSerializer(nested=True, many=True, required=False, allow_null=True)
    contracts_consume = ContractSerializer(nested=True, many=True, required=False, allow_null=True)

    class Meta:
        model = models.epg_model.EndPointGroup
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',            
            'created',
            'last_updated',
            'url',
            'applicationprofile',
            'domains',
            'contracts_provide',
            'contracts_consume',
            'bridgedomain',
            'subnets',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class EndPointSecurityGroupSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:endpointsecuritygroup-detail',
    )

    contracts_provide = ContractSerializer(nested=True, many=True, required=False, allow_null=True)
    contracts_consume = ContractSerializer(nested=True, many=True, required=False, allow_null=True)
    epgs_selector = EndPointGroupSerializer(nested=True, many=True, required=False, allow_null=True)
    ip_subnets_selector = PrefixSerializer(nested=True, many=True, required=False, allow_null=True)
    tags_selector = TagSerializer(nested=True, many=True, required=False, allow_null=True)
    vrf = VRFSerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = models.esg_model.EndPointSecurityGroup
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',            
            'vrf',
            'created',
            'last_updated',
            'url',
            'applicationprofile',
            'contracts_provide',
            'contracts_consume',
            'epgs_selector',
            'ip_subnets_selector',
            'tags_selector',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class DomainSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:domain-detail',
    )

    vlan_pool = VLANGroupSerializer(nested=True, required=False, allow_null=True)

    class Meta:
        model = models.domain_model.Domain
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'domain_type',          
            'vlan_pool',
            'pool_allocation_mode',
            'created',
            'last_updated',
            'url',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class AAEPSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:aaep-detail',
    )

    domains = VLANGroupSerializer(nested=True, many=True, required=False, allow_null=True)

    class Meta:
        model = models.aaep_model.AAEP
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',            
            'created',
            'last_updated',
            'url',
            'infrastructure_vlan',
            'domains',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class AAEPStaticBindingSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:aaepstaticbinding-detail',
    )

    aaep = AAEPSerializer(nested=True)
    tenant = TenantSerializer(nested=True)
    applicationprofile = ApplicationProfileSerializer(nested=True)
    epg = EndPointGroupSerializer(nested=True)

    class Meta:
        model = models.aaep_model.AAEPStaticBinding
        fields = (
            'id',
            'aaep',
            'created',
            'last_updated',
            'tenant',
            'applicationprofile',
            'epg',
            'encap',
            'mode',
        )
        brief_fields = ("id", "url")


class LinkLevelSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:linklevel-detail',
    )

    class Meta:
        model = models.policy_link_level_model.LinkLevel
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'created',
            'last_updated',
            'url',
            'negotiation',
            'speed',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class PolicyGroupSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:policygroup-detail',
    )

    aaep = AAEPSerializer(nested=True)
    linklevel = LinkLevelSerializer(nested=True)

    class Meta:
        model = models.ipg_model.PolicyGroup
        fields = (
            'id',
            'display',
            'name',
            'slug',
            'description',
            'created',
            'last_updated',
            'url',
            'aaep',
            'linklevel',
        )
        brief_fields = ("id", "display", "name", "slug", "url")


class PolicyGroupAssignementSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_aci-api:policygroupassignement-detail',
    )

    ipg = PolicyGroupSerializer(nested=True)
    device = DeviceSerializer(nested=True)
    interface = InterfaceSerializer(nested=True)

    class Meta:
        model = models.ipg_model.PolicyGroupAssignement
        fields = (
            'id',
            'ipg',
            'device',
            'interface',
            'created',
            'last_updated',            
        )
        brief_fields = ("id", "url")



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin

from .. import tables
from .. models import contract_filter_model
from .. forms import contract_filter_form

__all__ = (
    "ContractFilterView",
    "ContractFilterListView",
    "ContractFilterEditView",
    "ContractFilterDeleteView",
    "ContractFilterBulkDeleteView",
)


class ContractFilterView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = contract_filter_model.ContractFilter.objects.all()

    def get_extra_context(self, request, instance):

        filterentry = instance.entry_filter.all()
        filter_entry_table = tables.ContractFilterEntryTable(filterentry)

        return {
            'filter_entry_table': filter_entry_table,
            'related_models': self.get_related_models(request, instance),
        }


class ContractFilterListView(generic.ObjectListView):
    queryset = contract_filter_model.ContractFilter.objects.all()
    table = tables.ContractFilterTable


class ContractFilterEditView(generic.ObjectEditView):
    queryset = contract_filter_model.ContractFilter.objects.all()
    form = contract_filter_form.ContractFilterForm
    default_return_url = 'plugins:netbox_aci:contractfilter_list'


class ContractFilterDeleteView(generic.ObjectDeleteView):
    queryset = contract_filter_model.ContractFilter.objects.all()


class ContractFilterBulkDeleteView(generic.BulkDeleteView):
    queryset = contract_filter_model.ContractFilter.objects.all()
    table = tables.ContractFilterTable



"""
Define the django form elements for the user interface. 
"""

from django import forms
from ipam.models import IPAddress, VRF
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, SlugField, DynamicModelChoiceField
from utilities.forms.rendering import FieldSet
from .. models import bd_model, l3out_model


class BridgeDomainForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    subnets = forms.ModelMultipleChoiceField(
        queryset=IPAddress.objects.filter(custom_field_data={"gateway": True}),
        required=False
    )

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',            
            'vrf',
            'l2_unknown_unicast',
            'arp_flooding',
            'unicast_routing',
            'subnets',
            'l3outs',
        ),
    )

    class Meta:
        model = bd_model.BridgeDomain

        fields = (
            'name',
            'slug',
            'description',            
            'comments',
            'vrf',
            'l2_unknown_unicast',
            'arp_flooding',
            'unicast_routing',
            'subnets',
            'l3outs',
        )


class BridgeDomainFilterForm(NetBoxModelFilterSetForm):

    model = bd_model.BridgeDomain

    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False
    )

    l3outs = forms.ModelMultipleChoiceField(
        queryset=l3out_model.L3Out.objects.all(),
        required=False
    )



"""
Define the django form elements for the user interface. 
"""

from dcim.models import Device, Interface
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, SlugField, DynamicModelChoiceField
from utilities.forms.rendering import FieldSet
from .. models import ipg_model, aaep_model


class PolicyGroupForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'name',
            'slug',
            'description',
            'aaep',
            'linklevel',
        ),
    )

    class Meta:
        model = ipg_model.PolicyGroup
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'aaep',
            'linklevel',
        )


class PolicyGroupAssignementForm(NetBoxModelForm):

    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
    )

    interface = DynamicModelChoiceField(
        queryset=Interface.objects.all(),
        required=False,
        query_params={
            "device_id": "$device",
        },
    )

    fieldsets = (
        FieldSet(
            'ipg',
            'device',
            'interface',
        ),
    )

    class Meta:
        model = ipg_model.PolicyGroupAssignement
        fields = (
            'ipg',
            'device',
            'interface',
        )

    def clean(self):
        """
        Prevent duplicate entries
        """
        super().clean()

        ipg = self.cleaned_data.get("ipg")
        device = self.cleaned_data.get("device")
        interface = self.cleaned_data.get("interface")

        if ipg and device and interface:
            if ipg_model.PolicyGroupAssignement.objects.filter(ipg=ipg, device=device, interface=interface).exists():
                self.add_error("interface", "Duplicate entry")


class PolicyGroupFilterForm(NetBoxModelFilterSetForm):

    model = ipg_model.PolicyGroup

    aaep = DynamicModelChoiceField(
        queryset=aaep_model.AAEP.objects.all(),
        required=False
    )



"""
Define the django model.
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import ContractQoSClassChoices, ContractTargetDSCPChoices
from . default_model import ACIDefault
from . contract_model import Contract

__all__ = (
    "ContractSubject",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class ContractSubject(ACIDefault):
    """
    This class definition defines a Django model for a contract subject.
    """    
    name = models.CharField(
        verbose_name=('name'),
        max_length=50,
        unique=True,
        validators=[input_validation],
    )

    contract = models.ForeignKey(
        Contract,
        on_delete=models.CASCADE,
        related_name="subject_contract",
        blank=True,
        null=True,
    )

    target_dscp = models.CharField(
        choices=ContractTargetDSCPChoices,
        blank=True,
    )

    qos_priority = models.CharField(
        choices=ContractQoSClassChoices,
        blank=True,
    )

    apply_both_directions = models.BooleanField(
        default=True,
    )

    reverse_filter_ports = models.BooleanField(
        default=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Contract Subject"
        verbose_name_plural = "Contract Subjects"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:contractsubject', args=[self.pk])



"""
Define the logic of the plugin.
"""

from django.db.models import Q
from django.shortcuts import render
from django.views.generic import View
from ipam.models import vrfs
from netbox.views import generic
from utilities.views import GetRelatedModelsMixin, ViewTab, register_model_view
from .. import tables
from .. models import contract_model
from .. forms import contract_form

__all__ = (
    "ContractView",
    "ContractListView",
    "ContractEditView",
    "ContractDeleteView",
    "ContractBulkDeleteView",
    "ContractIpamVrfTabView",
)


class ContractView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = contract_model.Contract.objects.all()

    def get_extra_context(self, request, instance):

        contractsubject = instance.subject_contract.all()
        subject_table = tables.ContractSubjectTable(contractsubject)
        return {
            'subject_table': subject_table,
            'vrf_consume_table': instance.vrfs_consume.all(),
            'vrf_provide_table': instance.vrfs_provide.all(),
            'related_models': self.get_related_models(request, instance),
        }


class ContractListView(generic.ObjectListView):
    queryset = contract_model.Contract.objects.all()
    table = tables.ContractTable


class ContractEditView(generic.ObjectEditView):
    queryset = contract_model.Contract.objects.all()
    form = contract_form.ContractForm
    default_return_url = 'plugins:netbox_aci:contract_list'


class ContractDeleteView(generic.ObjectDeleteView):
    queryset = contract_model.Contract.objects.all()


class ContractBulkDeleteView(generic.BulkDeleteView):
    queryset = contract_model.Contract.objects.all()
    table = tables.ContractTable


@register_model_view(vrfs.VRF, "vzany")
class ContractIpamVrfTabView(View):
    tab = ViewTab(
        label="ACI vzAny Assignements",
        badge=lambda obj: contract_model.Contract.objects.filter(
            Q(vrfs_consume=obj) |
            Q(vrfs_provide=obj)
        ).count(),
        hide_if_empty = True,
        permission="netbox_aci.view_contract",
    )

    def get(self, request, pk):
        vrf = vrfs.VRF.objects.get(pk=pk)
        contract_consume = contract_model.Contract.objects.filter(vrfs_consume=vrf)
        contract_provide = contract_model.Contract.objects.filter(vrfs_provide=vrf)

        return render(
            request,
            "netbox_aci/contract_tab.html",
            context={
                "tab": self.tab,
                "object": vrf,
                "vrf_consume_table": contract_consume,
                "vrf_provide_table": contract_provide,
            },
        )



"""
Define the django model.
"""

from django.core.validators import RegexValidator
from django.urls import reverse
from django.db import models
from .. choices import LinkLevelNegotiationChoices, LinkLevelSpeedChoices
from . default_model import ACIDefault

__all__ = (
    "LinkLevel",
)

input_validation = RegexValidator(
    r"^[a-zA-Z0-9-_]+$",
    "Only alphanumeric, hyphens, and underscores are allowed.",
)


class LinkLevel(ACIDefault):
    """
    This class definition defines a Django model for an Interface LinkLevel Policy.
    """
    #Fields
    name = models.CharField(
        verbose_name=('name'),
        max_length=100,
        unique=True,
        validators=[input_validation],
    )

    negotiation = models.CharField(
        choices=LinkLevelNegotiationChoices,
        blank=True,
    )

    speed = models.CharField(
        choices=LinkLevelSpeedChoices,
        blank=True,
    )

    #Metadata
    class Meta:
        ordering = ["name"]
        verbose_name = "Link Level Policy"
        verbose_name_plural = "Link Level Policies"

    #Methods
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_aci:linklevel', args=[self.pk])



"""
Define the django form elements for the user interface. 
"""

from django import forms
from ipam.models import VRF, Prefix
from extras.models import Tag
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, SlugField, DynamicModelChoiceField
from utilities.forms.rendering import FieldSet, TabbedGroups
from .. models import esg_model, ap_model, contract_model, epg_model


class EndPointSecurityGroupForm(NetBoxModelForm):

    slug = SlugField()
    comments = CommentField()

    fieldsets = (
        FieldSet(
            TabbedGroups(
                FieldSet(
                    'name',
                    'slug',
                    'description',
                    'applicationprofile',
                    'vrf',
                    'contracts_consume',
                    'contracts_provide',
                    name = "ESG",
                ),
                FieldSet(
                    'epgs_selector',
                    'ip_subnets_selector',
                    'tags_selector',
                    name = "Selectors"
                ),
            ),
        ),
    )

    class Meta:
        model = esg_model.EndPointSecurityGroup
        fields = (
            'name',
            'slug',
            'description',
            'comments',
            'applicationprofile',
            'vrf',
            'contracts_consume',
            'contracts_provide',
            'epgs_selector',
            'ip_subnets_selector',
            'tags_selector',
        )


class EndPointSecurityGroupFilterForm(NetBoxModelFilterSetForm):

    model = esg_model.EndPointSecurityGroup

    applicationprofile = DynamicModelChoiceField(
        queryset=ap_model.ApplicationProfile.objects.all(),
        required=False
    )

    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False
    )

    contracts_consume = forms.ModelMultipleChoiceField(
        queryset=contract_model.Contract.objects.all(),
        required=False
    )

    contracts_provide = forms.ModelMultipleChoiceField(
        queryset=contract_model.Contract.objects.all(),
        required=False
    )

    epgs_selector = forms.ModelMultipleChoiceField(
        queryset=epg_model.EndPointGroup.objects.all(),
        required=False
    )

    ip_subnets_selector = forms.ModelMultipleChoiceField(
        queryset=Prefix.objects.all(),
        required=False
    )

    tags_selector = forms.ModelMultipleChoiceField(
        queryset=Tag.objects.all(),
        required=False
    )



"""
Define the logic of the plugin.
"""

from netbox.views import generic
from utilities.views import GetRelatedModelsMixin
from .. import tables
from .. models import contract_subject_model
from .. forms import contract_subject_form

__all__ = (
    "ContractSubjectView",
    "ContractSubjectListView",
    "ContractSubjectEditView",
    "ContractSubjectDeleteView",
    "ContractSubjectBulkDeleteView",
)


class ContractSubjectView(GetRelatedModelsMixin, generic.ObjectView):
    queryset = contract_subject_model.ContractSubject.objects.all()

    def get_extra_context(self, request, instance):

        subjectfilter = instance.filter_subject.all()
        filter_table = tables.ContractFilterTable(subjectfilter)

        return {
            'filter_table': filter_table,
            'related_models': self.get_related_models(request, instance),
        }


class ContractSubjectListView(generic.ObjectListView):
    queryset = contract_subject_model.ContractSubject.objects.all()
    table = tables.ContractSubjectTable


class ContractSubjectEditView(generic.ObjectEditView):
    queryset = contract_subject_model.ContractSubject.objects.all()
    form = contract_subject_form.ContractSubjectForm
    default_return_url = 'plugins:netbox_aci:contractsubject_list'


class ContractSubjectDeleteView(generic.ObjectDeleteView):
    queryset = contract_subject_model.ContractSubject.objects.all()


class ContractSubjectBulkDeleteView(generic.BulkDeleteView):
    queryset = contract_subject_model.ContractSubject.objects.all()
    table = tables.ContractSubjectTable
